unit UGlobals;

interface

uses graphics;
type
  Trgb = (cr,cg,cb);

  Topt = (esc, bok, ToAVR, ToDSP);


TCIVsubcomtype =
(
	SUBCOM_NONE,
	SUBCOM_HEX,
	SUBCOM_HEX_AND_LEVEL,
	SUBCOM_2BCD,
	SUBCOM_3BCD
);


TCIVreadsend =
(
	SR_READSEND,
	SR_SEND
);


TCIVencmode =
(
	ENC_NODATA,
	ENC_ONOFF,
	ENC_UPDOWN,
	ENC_TOGGLE
);



TCIV = record
          cvRigAddress: byte;
			    cvEncatAddress: byte;
          cvSubComType: TCIVsubcomtype;
          cvReadSend: TCIVreadsend;
			    cvEncMode: TCIVencmode;
			    cvDataDigits: byte;
			    cvCommand: byte;       // hex
			    cvSubComHex: byte;
			    cvLevelData: byte;    /// for command 0x14
			    cvSubComDecimal: longword; // will be converted to bcd
			    cvSpare: array[0..14] of byte;
        end;

TEnconfig =
packed record
  ecActive: byte;
  ecNumber: byte;
  ecStep: word;
  ecMin: longint;
  ecMax: longint;
  ecReadBytes: byte;
  ecSetBytes: byte;
  ecAnswerBytes: byte;
  ecReadMask: array[0..15] of char;
  ecSetMask: array[0..27] of char;

  ecDescription: array[0..20] of char;
  ecUnits: array[0..6] of char;
  // BaudRate, StopBits, Parity, CharBits set only in record for dummy Encoder eight
  ecBaudRate: byte;   // index of combo
  ecStopBits: byte;  // index of combo
  ecParity: byte;     // index of combo
  ecCharBits: byte;    // index of combo
  ecDisplayPanel: byte; // 0 = no panel
  ecDisplayMult: byte;
  ecDisplayDiv: byte;
  ecDecPlaces: byte;
  ecLookup: byte;
  ecContrast: byte;  // 0 to 255    dummy encoder 8 only
  ecIcom: byte;
  ecSpare: array[0..1] of byte;
  case integer of
    0: ( ecAnswerMask: array[0..27] of char;);      // overlays
    1: ( ecCIV: TCIV;);
end;

TLookupInfo = record
  nfTitle: array[0..26] of char;
  nfSpare: array[0..4] of byte;
end;



const

   clPaleRed:    Tcolor = $00E0E0FF;
   clPaleBlue:   Tcolor = $00FFE0E0;
   clPaleGreen:  Tcolor = $00E0FFE0;
   clPaleYellow: Tcolor = $00E0FFFF;
   clDeepYellow: Tcolor = $0000E8E8;
   clPaleBrown:  Tcolor = $00E8F0FF;


   cBlack:    string[7] = '\cf0 ';
   cRed:      string[7] = '\cf1 ';
   cGreen:    string[7] = '\cf2 ';
   cBlue:     string[7] = '\cf3 ';
   cTeal:     string[7] = '\cf4 ';
   cBrown:    string[7] = '\cf5 ';
   cMagenta:  string[7] = '\cf6 ';
   cEmerald:  string[7] = '\cf7 ';
   cNavy:     string[7] = '\cf8 ';

   g_ColourDefs: array[1..9,cr..cb] of integer
    =  ( (140,000,000),   // red
         (000,090,000),   // green
         (000,000,130),   // blue
         (000,080,080),   // teal
         (100,050,020),   // brown
         (120,000,90),    // magenta
         (000,100,050),   // emerald
         (000,000,060),   // navy
         (000,000,000) );



const g_PowerLabels: array[0..4,1..3] of string =
   (('2.5', '5.0', '7.5'),
		('5',   '10',  '15'),
		('10',  '20',  '30'),
		('25',  '50',  '75'),
		('50',  '100', '150'));


var
   cHeader:       string[7];
   cHeading1:     string[7];
   cHeading2:     string[7];
   cAdc:          string[7];
   cWatts:        string[7];
   cLedNo:        string[7];

   g_Lookups: array[0..7,0..53] of word;

   g_LookupInfo: TLookupInfo;

  g_HobcatVersion: string;

  g_TextFile: Text;
  g_TextFilePath: string;
  g_tempPath: string;

  g_XcatVersion: string;

  g_UsbReady: boolean;

  HcerrFilename: string;
  HcerrFile: Text;
  TelemMode: char;

  g_Encoders: array[0..8] of TEnconfig;

procedure StringToCharray(s: string; p: PChar);
function HexToInt(s: string; var LWL, LWH: longword): boolean;
function HexTest(s: string; nc: byte): boolean;

implementation

uses UIniData, SysUtils, Dialogs;



procedure StringToCharray(s: string; p: PChar);
var
  nc,i: integer;
begin
  nc := Length(s);
  if nc > 0 then
  begin
    for i := 0 to nc-1 do p[i] := s[i+1];
  end;
  p[nc] := #0;
end;




function HexToInt(s: string; var LWL, LWH: longword): boolean;
var
  nc,i: byte;
  su: string;
  v: byte;
begin
  su := uppercase(s);
  nc := length(su);
  LWL := 0;
  LWH := 0;
  result := false;
  if nc = 0 then exit;
  for i := 1 to nc do
  begin
    if (i+7) < nc then LWH := LWH*16 else LWL := LWL*16;
    v := ord(su[i]);
    if (v < $30) OR (v > $46) OR ( (v >$39) AND (v < $41)) then exit;
    if v > $39 then v := v - 7;
    v := v - $30;
    if (i + 7) < nc then LWH :=  LWH + v else LWL := LWL + v;
  end;
  result := true;
end;


function HexTest(s: string; nc: byte): boolean;
var
  n: byte;
  ch: char;
  i: byte;
  ok: boolean;
begin
  result := false;
  n := Length(s);
  if (n=0) OR (n > nc) then exit;
  for i := 1 to n do
  begin
    ch := s[i];
    ok := false;
    if (ch >= '0') AND (ch <= '9') then Ok := true;
    if (ch >= 'A') AND (ch <= 'F') then Ok := true;
    if not ok then exit;
  end;
  result := true;
end;


end.
