unit UIniData;

interface

uses Inifiles,SysUtils, graphics, forms,  Registry, Windows,
     Dialogs, UGlobals, UfrmEnconfig, UfrmLookups;

type

  RegIniData_type
    = record
        // Registry based INIdata
        IniMainFormLeft: integer;
        IniMainFormTop: integer;
        IniMainFormWidth: integer;
        IniMainFormHeight: integer;

        IniDebugIdColWidthA: integer;
        IniDebugVarNameColWidthA: integer;
        IniDebugVarNameColWidthB: integer;
        IniDebugSplitterPosition: integer;

        IniDebugAutoVars: boolean;
        IniDebugAutoPorts: boolean;
        IniDebugRefresh: integer;
        IniDebugViewPorts: boolean;
        IniDebugViewVars: boolean;

        IniEncodersIniFileName: string;
        IniLookupsIniFileName: string;
      end;


{

TEnconfig = record
  ecActive: byte;
  ecNumber: byte;
  ecStep: word;
  ecMin: longint;
  ecMax: longint;
  ecReadBytes: byte;
  ecSetBytes: byte;
  ecAnswerBytes: byte;
  ecReadMask: array[0..15] of char;
  ecSetMask: array[0..27] of char;
  ecAnswerMask: array[0..27] of char;
  ecTitle: array[0..23] of char;
  ecUnits: array[0..6] of char;
  // BaudRate, StopBits, Parity, CharBits set only in record for Encoder zero 
  ecBaudRate: byte;   // index of combo
  ecStopBits: byte;  // index of combo
  ecParity: byte;     // index of combo
  ecCharBits: byte;    // index of combo
  ecSpare: array[0..5] of byte;
end;
}


var
  g_RegIniData: RegIniData_type;
  RegIniFile: TRegIniFile;

  EnconfigIniFile: TIniFile;
  EnconfigIniFileName: string;

  LookupsIniFile: TIniFile;
  LookupsIniFileName: string;

  function RegIniOpen: boolean;
  procedure EnsureIniFileIsRegistry;


  procedure SaveRegIniFile;
  procedure LoadRegInifile;
  procedure SaveDebugIniData;
  procedure LoadDebugIniData;
  procedure LoadMainCoordinates;
  procedure SaveMainCoordinates;
  procedure LoadDebugCoordinates;
  procedure SaveDebugCoordinates;

  procedure ReadFromEnconfigInifile;
  procedure WriteToEnconfigIniFile;

  procedure ReadFromLookupsInifile;
  procedure WriteToLookupsIniFile;



implementation

uses UfrmMain;

const
 KeyName: string = 'Software\hbradios\xCAT';


procedure EnsureIniFileIsRegistry;
begin
  try
    RegIniFile := TRegIniFile.Create;
    RegIniFile.RootKey := HKEY_CURRENT_USER; // Section to look for within the registry
    if not RegIniFile.OpenKey(KeyName,False) then
    begin
      RegIniFile.CreateKey(KeyName);
      RegIniFile.Free;
      Exit;
    end;
  except
    MessageDlg('Failed to create HKEY_CURRENT_USER Registry Key. INI settings will not be saved.',mtWarning,[mbOk],0);
  end;
  RegIniFile.Free;
end;


function RegIniOpen: boolean;
begin
  result := true;
  try
    RegIniFile := TRegIniFile.Create;
    RegIniFile.RootKey := HKEY_CURRENT_USER; // Section to look for within the registry
    RegIniFile.OpenKey(KeyName,true);
  except
    result := false;
    RegIniFile.Free;
  end;
end;




function BoolToInt(b: boolean): integer;
begin
  if b then result := 1 else result := 0;
end;


function IntToBool(i: integer): boolean;
begin
  if i=0 then result := FALSE else result := TRUE;
end;




procedure LoadDebugCoordinates;
var
  s: string;
  a: integer;
begin
  s := ExtractFilePath(Application.ExeName);
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    IniDebugIdColWidthA := ReadInteger('DebugForm','DebugIdColWidthA',25);
    IniDebugVarNameColWidthA := ReadInteger('DebugForm','DebugVarNameColWidthA',95);
    IniDebugVarNameColWidthB := ReadInteger('DebugForm','DebugVarNameColWidthB',56);
    IniDebugSplitterPosition := ReadInteger('DebugForm','DebugSplitterPosition',360);

  end;
  RegIniFile.Free;
end;



procedure SaveDebugCoordinates;
begin
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    WriteInteger('DebugForm','DebugIdColWidthA',IniDebugIdColWidthA);
    WriteInteger('DebugForm','DebugVarNameColWidthA',IniDebugVarNameColWidthA);
    WriteInteger('DebugForm','DebugVarNameColWidthB',IniDebugVarNameColWidthB);
    WriteInteger('DebugForm','DebugSplitterPosition',IniDebugSplitterPosition);
  end;
  RegIniFile.Free;
end;


procedure SaveRegIniFile;
begin
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    WriteString('EncoderData','LastIniFileName',IniEncodersIniFileName);
  end;
  RegIniFile.Free;
end;


procedure LoadDebugIniData;
var
  s: string;
begin
  s := ExtractFilePath(Application.ExeName);
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    IniDebugAutoPorts := IntToBool(ReadInteger('DebugForm','DebugAutoPorts',0));
    IniDebugAutoVars := IntToBool(ReadInteger('DebugForm','DebugAutoVars',0));
    IniDebugRefresh := ReadInteger('DebugForm','DebugRefresh',20);
    IniDebugViewVars := IntToBool(ReadInteger('DebugForm','DebugViewVars',1));
    IniDebugViewPorts := IntToBool(ReadInteger('DebugForm','DebugViewPorts',1));
  end;
  RegIniFile.Free;
end;


procedure SaveDebugIniData;
begin
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    WriteInteger('DebugForm','DebugAutoPorts',BoolToInt(IniDebugAutoPorts));
    WriteInteger('DebugForm','DebugAutoVars',BoolToInt(IniDebugAutoVars));
    WriteInteger('DebugForm','DebugRefresh',IniDebugRefresh);
    WriteInteger('DebugForm','DebugViewPorts',BoolToInt(IniDebugViewPorts));
    WriteInteger('DebugForm','DebugViewVars',BoolToInt(IniDebugViewVars));
  end;
  RegIniFile.Free;
end;


procedure LoadRegIniFile;
var
  s: string;
begin
  s := ExtractFilePath(Application.ExeName);
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    IniEncodersIniFileName := ReadString('EncoderData','LastIniFileName','');

  end;
  RegIniFile.Free;
end;


procedure LoadMainCoordinates;
begin
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    IniMainFormTop := ReadInteger('coordinates','mainformtop',30);
    IniMainFormLeft := ReadInteger('coordinates','mainformleft',30);
    IniMainFormWidth := ReadInteger('coordinates','mainformwidth',400);
    IniMainFormHeight := ReadInteger('coordinates','mainformheight',300);
  end;
  RegIniFile.Free;

  with g_RegIniData do begin
    if IniMainFormLeft > Screen.Width-100
    then begin
      IniMainFormLeft := Screen.Width - 100;
      SaveMainCoordinates;
    end;
    if IniMainFormTop > Screen.Height-100
    then begin
      IniMainFormTop := Screen.Height - 100;
      SaveMainCoordinates;
    end;
    if IniMainFormHeight > Screen.Height
    then begin
      IniMainFormLeft := Screen.Height - 60;
      SaveMainCoordinates;
    end;
    if IniMainFormTop > Screen.Width
    then begin
      IniMainFormLeft := Screen.Width - 60;
      SaveMainCoordinates;
    end;
  end;
end;


procedure SaveMainCoordinates;
begin
  if not RegIniOpen then exit;
  with g_RegIniData, RegIniFile do begin
    WriteInteger('coordinates','mainformtop',IniMainFormTop);
    WriteInteger('coordinates','mainformleft',IniMainFormLeft);
    WriteInteger('coordinates','mainformheight',IniMainFormHeight);
    WriteInteger('coordinates','mainformwidth',IniMainFormWidth);
  end;
  RegIniFile.Free;
end;

//////////////////////////////////////////////////////////

// ENCODER INIFILE - saved in INI format so as to readable / editable

procedure WriteToEnconfigIniFile;
var
  chb: char;
  e: byte;
  filename: string;
  header: string;
  nc: byte;
  tf: byte;
  carray: array[0..99] of char;
begin
  header := frmEnconfig.RzEdit_Title.text;
  filename := EnconfigIniFileName;
  nc := Length(header);
  StringToCharray(filename, carray);
  DeleteFile(carray);
  EnconfigIniFile := TIniFile.Create(FileName);
  with EnconfigIniFile do
  begin
    if nc > 0 then
    begin
       WriteString('EncoderCAT configuration','title',header);
    end;
    // the RS232 setting are handled as Itemidex apart from in this ini file
    WriteString('RS232','** *','  Baudrate, stopbits, parity and bits/char are character strings');
    WriteString('RS232','****','  and much match exactly the drop-selector-list items.');
    // should be writing data here from g_Encoders[8]
    // but call to this WriteToEnconfigIniFile is always preceded
    // by GetEncatOptions which sets the form from g_Encoders[8]
    WriteString('RS232','BaudRate',frmEnconfig.RzCombo_BaudRate.Text);
    WriteString('RS232','StopBits',frmEnconfig.RzCombo_StopBits.Text);
    WriteString('RS232','Parity',frmEnconfig.RzCombo_Parity.Text);
    WriteString('RS232','Bits/Char',frmEnconfig.RzCombo_CharBits.Text);
    if frmEnconfig.RzChBox_DisplayPanel.Checked then tf := 1 else tf := 0;
    WriteInteger('Display_panel','installed',tf);
    WriteInteger('Display_panel','contrast',trunc(frmEnconfig.RzSpinEdit_Contrast.Value));
    if frmEnconfig.RzRB_CIV.Checked then tf := 1 else tf := 0;
    WriteInteger('Grid','RS232orCIV',tf);

    for e := 0 to 7 do
    begin
      with g_Encoders[e] do
      begin
        chb := chr(48 + e);
        WriteInteger('Encoder ' + chb,'Active', ecActive);
        WriteInteger('Encoder ' + chb,'Number', ecNumber);
        WriteInteger('Encoder ' + chb,'Step', ecStep);
        WriteInteger('Encoder ' + chb,'Min', ecMin);
        WriteInteger('Encoder ' + chb,'Max', ecMax);
        WriteString('Encoder ' + chb,'Description', ecDescription);
        WriteString('Encoder ' + chb,'Units', ecUnits);
        WriteInteger('Encoder ' + chb,'DisplayMultiplier', ecDisplayMult);
        WriteInteger('Encoder ' + chb,'DisplayDivisor', ecDisplayDiv);
        WriteInteger('Encoder ' + chb,'DecimalPlaces', ecDecPlaces);
        WriteInteger('Encoder ' + chb,'Lookup', ecLookup);

        if(frmEnconfig.IcomCIV) then
        begin
          WriteString('Encoder ' + chb,'RigAddress', IntToHex(ecCIV.cvRigAddress, 2) + 'h');
          WriteString('Encoder ' + chb,'EncatAddress', IntToHex(ecCIV.cvEncatAddress, 2) + 'h');
          WriteInteger('Encoder ' + chb,'SubComType', byte(ecCIV.cvSubComType));
          WriteInteger('Encoder ' + chb,'ReadSend', byte(ecCIV.cvReadSend));
          WriteInteger('Encoder ' + chb,'EncMode', byte(ecCIV.cvEncMode));
          WriteInteger('Encoder ' + chb,'DataDigits', ecCIV.cvDataDigits);
          WriteString('Encoder ' + chb,'Command', IntToHex(ecCIV.cvCommand, 2) + 'h');
          WriteString('Encoder ' + chb,'SubComHex', IntToHex(ecCIV.cvSubComHex, 2) + 'h');
          WriteInteger('Encoder ' + chb,'Leveldata', ecCIV.cvLevelData);
          WriteInteger('Encoder ' + chb,'SubComDecimel', ecCIV.cvSubComDecimal);
        end
        else begin
          WriteInteger('Encoder ' + chb,'Read_chars', ecReadBytes);
          WriteInteger('Encoder ' + chb,'Set_chars', ecSetBytes);
          WriteInteger('Encoder ' + chb,'Answer_chars', ecAnswerBytes);
          WriteString('Encoder ' + chb,'Read_mask', ecReadMask);
          WriteString('Encoder ' + chb,'Set_mask', ecSetMask);
          WriteString('Encoder ' + chb,'Answer_mask', ecAnswerMask);
        end;
      end;
    end;
    Free;
  end;
end;


procedure ReadFromEnconfigInifile;
var
  s: string;
  chb: char;
  tf: byte;
  e: byte;
  nc: byte;
  i: byte;
  b: byte;
  LWL, LWH: longword;
  filename: string;
begin
  filename := EnconfigIniFileName;
  EnconfigIniFile := TIniFile.Create(filename);
  with EnconfigIniFile do
  begin
    s := ReadString('EncoderCAT configuration','title','');
    nc := Length(s);
    if nc > 0 then
    begin
      StringToCharray(s, g_Encoders[8].ecSetMask);   // 'Encoder 8' SetMask field used for title
    end;
    // the RS232 setting are handled as Itemidex apart from in this ini file
    with frmEnconfig, g_Encoders[8] do
    begin
      s := ReadString('RS232','BaudRate','38400');
      ecBaudRate := RzCombo_BaudRate.IndexOf(s);
      s := ReadString('RS232','StopBits','1');
      ecStopBits := RzCombo_StopBits.IndexOf(s);
      s := ReadString('RS232','Parity','none');
      ecParity := RzCombo_Parity.IndexOf(s);
      s := ReadString('RS232','Bits/Char','8');
      ecCharBits := RzCombo_CharBits.IndexOf(s);
      ecDisplayPanel := ReadInteger('Display_panel','installed',0);
      ecContrast := ReadInteger('DisplayPanel','contrast',64);
      ecIcom := ReadInteger('Grid','RS232orCIV',0);
    end;
    //////////////////////////////
    for e := 0 to 7 do
    begin
      chb := chr(48 + e);
      with g_Encoders[e] do
      begin
        ecActive := ReadInteger('Encoder ' + chb,'Active',0);
        ecNumber := ReadInteger('Encoder ' + chb,'Number',0);
        ecStep := ReadInteger('Encoder ' + chb,'Step',0);
        ecMin := ReadInteger('Encoder ' + chb,'Min',0);
        ecMax := ReadInteger('Encoder ' + chb,'Max',0);
        ecReadBytes := ReadInteger('Encoder ' + chb,'Read_chars',0);
        ecSetBytes := ReadInteger('Encoder ' + chb,'Set_chars',0);
        ecAnswerBytes := ReadInteger('Encoder ' + chb,'Answer_chars',0);
        StringToCharray(ReadString('Encoder ' + chb,'Read_mask',''), ecReadMask);
        StringToCharray(ReadString('Encoder ' + chb,'Set_mask',''), ecSetMask);
        StringToCharray(ReadString('Encoder ' + chb,'Answer_mask',''), ecAnswerMask);
        StringToCharray(ReadString('Encoder ' + chb,'Description',''), ecDescription);
        StringToCharray(ReadString('Encoder ' + chb,'Units',''), ecUnits);
        ecDisplayMult := ReadInteger('Encoder ' + chb,'DisplayMultiplier',1);
        ecDisplayDiv := ReadInteger('Encoder ' + chb,'DisplayDivisor',1);
        ecDecPlaces := ReadInteger('Encoder ' + chb,'DecimalPlaces',0);
        ecLookup := ReadInteger('Encoder ' + chb,'Lookup',0);
        if g_Encoders[8].ecIcom > 0 then
        begin
          s := ReadString('Encoder ' + chb,'RigAddress', '70h');
          s := copy(s, 1 ,2);
          HexToInt(s, LWL, LWH);
          b := LWL;
          ecCIV.cvRigAddress := b;

          s :=  ReadString('Encoder ' + chb,'EncatAddress', 'E0h');
          s := copy(s, 1 ,2);
          HexToInt(s, LWL, LWH);
          b := LWL;
          ecCIV.cvEncatAddress := b;

          ecCIV.cvSubComType := TCIVsubcomtype(ReadInteger('Encoder ' + chb,'SubComType', 4));
          ecCIV.cvReadSend := TCIVreadsend(ReadInteger('Encoder ' + chb,'ReadSend', 2));
          ecCIV.cvEncMode := TCIVencmode(ReadInteger('Encoder ' + chb,'EncMode', 2));
          ecCIV.cvDataDigits := ReadInteger('Encoder ' + chb,'DataDigits', 3);

          s := ReadString('Encoder ' + chb,'Command', '1Ah');
          s := copy(s, 1 ,2);
          HexToInt(s, LWL, LWH);
          b := LWL;
          ecCIV.cvCommand := b;

          s := ReadString('Encoder ' + chb,'SubComHex', '00h');
          s := copy(s, 1 ,2);
          HexToInt(s, LWL, LWH);
          b := LWL;
          ecCIV.cvSubComHex := b;

          ecCIV.cvLevelData := ReadInteger('Encoder ' + chb,'Leveldata', 0);
          ecCIV.cvSubComDecimal := ReadInteger('Encoder ' + chb,'SubComDecimel', 50114);
        end
        else begin
          ecReadBytes := ReadInteger('Encoder ' + chb,'Read_chars',0);
          ecSetBytes := ReadInteger('Encoder ' + chb,'Set_chars',0);
          ecAnswerBytes := ReadInteger('Encoder ' + chb,'Answer_chars',0);
          StringToCharray(ReadString('Encoder ' + chb,'Read_mask',''), ecReadMask);
          StringToCharray(ReadString('Encoder ' + chb,'Set_mask',''), ecSetMask);
          StringToCharray(ReadString('Encoder ' + chb,'Answer_mask',''), ecAnswerMask);
        end;
      end;
    end;

    Free;
  end;
end;

////////////////////////////////////////////////


procedure WriteToLookupsIniFile;
var
  che: char;
  i, e: byte;
  si: string;
  filename: string;
  header: string;
  nc: byte;
  tf: byte;
  carray: array[0..99] of char;
begin
  header := frmLookups.RzEdit_Title.text;
  filename := LookupsIniFileName;
  nc := Length(header);
  StringToCharray(filename, carray);
  DeleteFile(carray);
  LookupsIniFile := TIniFile.Create(FileName);
  with LookupsIniFile do
  begin
    if nc > 0 then
    begin
       WriteString('EncoderCAT lookups','title',header);
    end;
    for e := 0 to 7 do
    begin
      che := chr(48 + e);
      for i := 0 to 53 do
      begin
        si := IntToStr(i);
        if i < 10 then si := '0' + si;
        si := 'cat' + si;
        WriteInteger('Encoder' + che, si, g_Lookups[e, i]);
      end;
    end;
    Free;
  end;
end;


procedure ReadFromLookupsInifile;
var
  s: string;
  si: string;
  che: char;
  tf: byte;
  e, i: byte;
  nc: byte;
  filename: string;
begin
  filename := LookupsIniFileName;
  LookupsIniFile := TIniFile.Create(filename);
  with LookupsIniFile do
  begin
    s := ReadString('EncoderCAT lookups','title','');
    nc := Length(s);
    if nc > 0 then
    begin
      StringToCharray(s, g_LookupInfo.nfTitle);
      frmLookups.RzEdit_Title.Text := s;
    end;
    /////////////////////
    s := ReadString('EncoderCAT lookups','title','');
    nc := Length(s);
    //////////////////////////////
    for e := 0 to 7 do
    begin
      che := chr(48 + e);
      for i := 0 to 53 do
      begin
        si := IntToStr(i);
        if i < 10 then si := '0' + si;
        si := 'cat' + si;
        g_Lookups[e, i] := ReadInteger('Encoder' + che, si, 0);
      end;
    end;
    Free;
  end;
end;

////////////////////////////////////////////////


end.

